@@ -3,9 +3,9 @@
  *
  * Implementation of the Tamper-Proof Device Filesystem.
  *
- * Copyright (C) 2005  NTT DATA CORPORATION
+ * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.0.1 2005/12/06
+ * Version: 1.1.3   2006/07/13
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -17,10 +17,10 @@
  *  SYAORAN stands for "Simple Yet All-important Object Realizing Abiding Nexus".
  *  SYAORAN is a filesystem for /dev with Mandatory Access Control.
  *
- *  /dev needs to be writable, but this means that files on /dev might be tampered.
- *  SYAORAN can restrict combinations of (filename, filetype) that the system can create.
- *  The filetype is one of directory, regular file, FIFO, unix domain socket,
- *  symbolic link, character device file, block device file.
+ *  /dev needs to be writable, but this means that files on /dev might be tampered with.
+ *  SYAORAN can restrict combinations of (pathname, attribute) that the system can create.
+ *  The attribute is one of directory, regular file, FIFO, UNIX domain socket,
+ *  symbolic link, character or block device file with major/minor device numbers.
  *
  *  You can use SYAORAN alone, but I recommend you to use with SAKURA and TOMOYO.
  */
@@ -28,6 +28,10 @@
 #ifndef _LINUX_SYAORAN_H
 #define _LINUX_SYAORAN_H
 
+#ifndef __user
+#define __user
+#endif
+
 /***** SYAORAN start. *****/
 
 #include <linux/version.h>
@@ -82,7 +86,11 @@ static struct dentry *lookup_create2(const char *name, struct dentry *base, int
 {
 	struct dentry *dentry;
 	const int len = name ? strlen(name) : 0;
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+	mutex_lock(&base->d_inode->i_mutex);
+#else
 	down(&base->d_inode->i_sem);
+#endif
 	dentry = lookup_one_len(name, base, len);
 	if (IS_ERR(dentry)) goto fail;
 	if (!is_dir && name[len] && !dentry->d_inode) goto enoent;
@@ -109,7 +117,11 @@ static int fs_mkdir(const char *pathname, struct dentry *base, int mode, uid_t u
 		}
 		dput(dentry);
 	}
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+	mutex_unlock(&base->d_inode->i_mutex);
+#else
 	up(&base->d_inode->i_sem);
+#endif
 	return error;
 }
 
@@ -136,7 +148,11 @@ static int fs_mknod(const char *filename, struct dentry *base, int mode, dev_t d
 		}
 		dput(dentry);
 	}
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+	mutex_unlock(&base->d_inode->i_mutex);
+#else
 	up(&base->d_inode->i_sem);
+#endif
 	return error;
 }
 
@@ -160,7 +176,11 @@ static int fs_symlink(const char *pathname, struct dentry *base, char *oldname,
 		}
 		dput(dentry);
 	}
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+	mutex_unlock(&base->d_inode->i_mutex);
+#else
 	up(&base->d_inode->i_sem);
+#endif
 	return error;
 }
 
@@ -409,9 +429,15 @@ static void MakeNode(struct dev_entry *entry, struct dentry *root)
 			struct dentry *new_base;
 			const int len = filename - name;
 			*filename = '\0';
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+			mutex_lock(&base->d_inode->i_mutex);
+			new_base = lookup_one_len(name, base, len);
+			mutex_unlock(&base->d_inode->i_mutex);
+#else
 			down(&base->d_inode->i_sem);
 			new_base = lookup_one_len(name, base, len);
 			up(&base->d_inode->i_sem);
+#endif
 			dput(base);
 			/*
 			if (IS_ERR(new_base)) {
@@ -475,7 +501,7 @@ static int Syaoran_Initialize(struct super_block *sb, void *data)
 	static int first = 1;
 	if (first) {
 		first = 0;
-		printk("SYAORAN: 1.0.1 2005/12/06\n");
+		printk("SYAORAN: 1.1.3   2006/07/13\n");
 	}
 	{
 		struct inode *inode = new_inode(sb);
@@ -539,18 +565,12 @@ static int GetLocalAbsolutePath(struct dentry *dentry, char *buffer, int buflen)
 {
 	char *start = buffer;
 	char *end = buffer + buflen;
-	char *retval;
 	int namelen;
 
 	if (buflen < 256) goto out;
-	
+
 	*--end = '\0';
 	buflen--;
-	
-	/* Get '/' right */
-	retval = end-1;
-	*retval = '/';
-	
 	for (;;) {
 		struct dentry *parent;
 		if (IS_ROOT(dentry)) break;
@@ -561,15 +581,15 @@ static int GetLocalAbsolutePath(struct dentry *dentry, char *buffer, int buflen)
 		end -= namelen;
 		memcpy(end, dentry->d_name.name, namelen);
 		*--end = '/';
-		retval = end;
 		dentry = parent;
 	}
+	if (*end == '/') { buflen++; end++; }
 	namelen = dentry->d_name.len;
 	buflen -= namelen;
 	if (buflen < 0) goto out;
-	retval -= namelen-1;/* hit the slash */
-	memcpy(retval, dentry->d_name.name, namelen);
-	memmove(start, retval, strlen(retval) + 1);
+	end -= namelen;
+	memcpy(end, dentry->d_name.name, namelen);
+	memmove(start, end, strlen(end) + 1);
 	return 0;
  out:
 	return -ENOMEM;
@@ -779,7 +799,11 @@ static int syaoran_create_tracelog(struct super_block *sb, const char *filename)
 		}
 		dput(dentry);
 	}
+#if LINUX_VERSION_CODE  >= KERNEL_VERSION(2,6,16)
+	mutex_unlock(&base->d_inode->i_mutex);
+#else
 	up(&base->d_inode->i_sem);
+#endif
 	dput(base);
 	return error;
 }
