@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.2   2006/06/02
+ * Version: 1.1.3   2006/07/13
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -125,7 +125,24 @@ int ReadCapabilityStatus(IO_BUFFER *head)
 
 /*************************  AUDIT FUNCTIONS  *************************/
 
-static int AuditCapabilityLog(const unsigned int capability, const int is_granted);
+#ifdef CONFIG_TOMOYO_AUDIT
+static int AuditCapabilityLog(const unsigned int capability, const int is_granted)
+{
+	char *buf;
+	int len;
+	struct timeval tv;
+	struct task_struct *task = current;
+	const char *domainname = task->domain_info->domainname;
+	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
+	do_gettimeofday(&tv);
+	len = strlen(domainname) + 256;
+	if ((buf = ccs_alloc(len)) == NULL) return -ENOMEM;
+	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n" KEYWORD_ALLOW_CAPABILITY "%s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, capability2keyword(capability));
+	return WriteAuditLog(buf, is_granted);
+}
+#else
+static inline void AuditCapabilityLog(const unsigned int capability, const int is_granted) {}
+#endif
 
 /*************************  CAPABILITY ACL HANDLER  *************************/
 
@@ -203,21 +220,4 @@ int AddCapabilityPolicy(char *data, struct domain_info *domain, const int is_del
 
 EXPORT_SYMBOL(CheckCapabilityACL);
 
-/*************************  AUDIT FUNCTIONS  *************************/
-
-static int AuditCapabilityLog(const unsigned int capability, const int is_granted)
-{
-	char *buf;
-	int len;
-	struct timeval tv;
-	struct task_struct *task = current;
-	const char *domainname = task->domain_info->domainname;
-	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
-	do_gettimeofday(&tv);
-	len = strlen(domainname) + 256;
-	if ((buf = kzalloc(len, GFP_KERNEL)) == NULL) return -ENOMEM;
-	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n" KEYWORD_ALLOW_CAPABILITY "%s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, capability2keyword(capability));
-	return WriteAuditLog(buf, is_granted);
-}
-
 /***** TOMOYO Linux end. *****/
