@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.2   2006/06/02
+ * Version: 1.1.3   2006/07/13
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -116,8 +116,40 @@ static int AddDoubleWriteACL(const unsigned int type_hash, const char *filename1
 
 /*************************  AUDIT FUNCTIONS  *************************/
 
-static int AuditFileLog(const char *filename, const unsigned short int perm, const int is_granted);
-static int AuditWriteLog(const char *operation, const char *filename1, const char *filename2, const int is_granted);
+#ifdef CONFIG_TOMOYO_AUDIT
+static int AuditFileLog(const char *filename, const unsigned short int perm, const int is_granted)
+{
+	char *buf;
+	const char *domainname = current->domain_info->domainname;
+	int len;
+	struct timeval tv;
+	struct task_struct *task = current;
+	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
+	do_gettimeofday(&tv);
+	len = strlen(domainname) + strlen(filename) + 256;
+	if ((buf = ccs_alloc(len)) == NULL) return -ENOMEM;
+	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n%d %s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, perm, filename);
+	return WriteAuditLog(buf, is_granted);
+}
+
+static int AuditWriteLog(const char *operation, const char *filename1, const char *filename2, const int is_granted)
+{
+	char *buf;
+	int len;
+	struct timeval tv;
+	struct task_struct *task = current;
+	const char *domainname = task->domain_info->domainname;
+	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
+	do_gettimeofday(&tv);
+	len = strlen(domainname) + strlen(operation) + strlen(filename1) + strlen(filename2) + 256;
+	if ((buf = ccs_alloc(len)) == NULL) return -ENOMEM;
+	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\nallow_%s %s %s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, operation, filename1, filename2);
+	return WriteAuditLog(buf, is_granted);
+}
+#else
+static inline void AuditFileLog(const char *filename, const unsigned short int perm, const int is_granted) {}
+static inline void AuditWriteLog(const char *operation, const char *filename1, const char *filename2, const int is_granted) {}
+#endif
 
 /*************************  GLOBALLY READABLE FILE HANDLER  *************************/
 
@@ -395,7 +427,7 @@ int CheckFilePerm(const char *filename, const unsigned short int perm, const int
 		const char *realname = realpath(filename);
 		if (realname) {
 			error = CheckFilePerm(realname, perm, 1, operation);
-			kfree(realname);
+			ccs_free(realname);
 		} else {
 			error = -ENOENT;
 		}
@@ -588,7 +620,7 @@ int CheckSingleWritePermission(const unsigned int operation, struct dentry *dent
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
 	if (CheckACLFlags(operation) < 0) return 0;	
-	buf = kzalloc(PAGE_SIZE, GFP_KERNEL);
+	buf = ccs_alloc(PAGE_SIZE);
 	if (buf == NULL) return -ENOMEM;
 	if ((error = realpath_from_dentry(dentry, mnt, buf, PAGE_SIZE - 4)) == 0) {
 		int len = strlen(buf);
@@ -615,7 +647,7 @@ int CheckSingleWritePermission(const unsigned int operation, struct dentry *dent
 	} else {
 		printk("DEBUG: realpath_from_dentry = %d\n", error);
 	}
-	kfree(buf);
+	ccs_free(buf);
 	if (!is_enforce) error = 0;
 	return error;
 }
@@ -628,10 +660,10 @@ int CheckDoubleWritePermission(const unsigned int operation, struct dentry *dent
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
 	if (CheckACLFlags(operation) < 0) return 0;		
-	buf1 = kzalloc(PAGE_SIZE, GFP_KERNEL);
-	buf2 = kzalloc(PAGE_SIZE, GFP_KERNEL);
+	buf1 = ccs_alloc(PAGE_SIZE);
+	buf2 = ccs_alloc(PAGE_SIZE);
 	if (!buf1 || !buf2) {
-		kfree(buf1); kfree(buf2);
+		ccs_free(buf1); ccs_free(buf2);
 		return -ENOMEM;
 	}
 	if ((error = realpath_from_dentry(dentry1, mnt1, buf1, PAGE_SIZE - 4)) == 0) {
@@ -668,8 +700,8 @@ int CheckDoubleWritePermission(const unsigned int operation, struct dentry *dent
 	} else {
 		printk("DEBUG: realpath_from_dentry = %d\n", error);
 	}
-	kfree(buf1);
-	kfree(buf2);
+	ccs_free(buf1);
+	ccs_free(buf2);
 	if (!is_enforce) error = 0;
 	return error;
 }
@@ -717,36 +749,4 @@ EXPORT_SYMBOL(CheckFilePerm);
 EXPORT_SYMBOL(CheckSingleWritePermission);
 EXPORT_SYMBOL(CheckDoubleWritePermission);
 
-/*************************  AUDIT FUNCTIONS  *************************/
-
-static int AuditFileLog(const char *filename, const unsigned short int perm, const int is_granted)
-{
-	char *buf;
-	const char *domainname = current->domain_info->domainname;
-	int len;
-	struct timeval tv;
-	struct task_struct *task = current;
-	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
-	do_gettimeofday(&tv);
-	len = strlen(domainname) + strlen(filename) + 256;
-	if ((buf = kzalloc(len, GFP_KERNEL)) == NULL) return -ENOMEM;
-	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n%d %s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, perm, filename);
-	return WriteAuditLog(buf, is_granted);
-}
-
-static int AuditWriteLog(const char *operation, const char *filename1, const char *filename2, const int is_granted)
-{
-	char *buf;
-	int len;
-	struct timeval tv;
-	struct task_struct *task = current;
-	const char *domainname = task->domain_info->domainname;
-	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
-	do_gettimeofday(&tv);
-	len = strlen(domainname) + strlen(operation) + strlen(filename1) + strlen(filename2) + 256;
-	if ((buf = kzalloc(len, GFP_KERNEL)) == NULL) return -ENOMEM;
-	snprintf(buf, len - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\nallow_%s %s %s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname, operation, filename1, filename2);
-	return WriteAuditLog(buf, is_granted);
-}
-
 /***** TOMOYO Linux end. *****/
